// $Id$
/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.annotatedlineeditor;

import org.apache.log4j.Logger;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.jface.text.BadLocationException;
import org.eclipse.jface.text.IDocument;
import org.eclipse.jface.text.source.ISourceViewer;
import org.eclipse.jface.text.source.IVerticalRuler;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.ui.IEditorInput;
import org.eclipse.ui.editors.text.TextEditor;

import eu.etaxonomy.cdm.api.conversation.ConversationHolder;
import eu.etaxonomy.cdm.api.conversation.IConversationEnabled;
import eu.etaxonomy.cdm.model.common.CdmBase;
import eu.etaxonomy.cdm.persistence.hibernate.CdmDataChangeMap;
import eu.etaxonomy.taxeditor.operations.IPostOperationEnabled;
import eu.etaxonomy.taxeditor.store.CdmStore;

/**
 * A list-based editor, where each line in the editor's document is associated with a domain object.
 * <p>
 * Extending classes must set:
 * <ul>
 * <li>an <code>IEntityCreator</code> to create a new domain object when the user creates a new line;
 * <li>an <code>IEntityPersistenceService</code> for interacting with the persistence layer; and
 * <li>an <code>ILineDisplayStrategy</code> for various visual manifestations of the domain object. 
 * </ul>
 * 
 * @author p.ciardelli
 * @created 25.06.2009
 * @version 1.0
 */
public class AnnotatedLineEditor extends TextEditor implements IConversationEnabled, IPostOperationEnabled {
	@SuppressWarnings("unused")
	private static final Logger logger = Logger
			.getLogger(AnnotatedLineEditor.class);
	
	private ConversationHolder conversation;
	
	private IEntityPersistenceService persistenceService;
	protected ILineDisplayStrategy lineDisplayStrategy;
	private IEntityCreatorService entityCreatorService;
	private IEntityCreator<?> entityCreator;
	
	/**
	 * When connected to a datasource, set conversation to CdmStore.NewTransactionalConversation().
	 * <p>
	 * To work offline, set conversation to NULL.
	 */
	public AnnotatedLineEditor(ConversationHolder conversation) {
		this.conversation = conversation;
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.ui.editors.text.TextEditor#doSetInput(org.eclipse.ui.IEditorInput)
	 */
	@Override
	protected void doSetInput(IEditorInput input) throws CoreException {

		AnnotatedLineDocumentProvider provider = createAnnotatedLineDocumentProvider();
		if (entityCreatorService != null) {
			provider.setEntityCreator(entityCreatorService.getEntityCreator(input), input);
		} else {
			provider.setEntityCreator(entityCreator, input);
		}
		provider.setPersistenceService(persistenceService, input);
		provider.setLineDisplayStrategy(lineDisplayStrategy, input);
		setDocumentProvider(provider);
		
		super.doSetInput(input);	
	}
		
	/**
	 * @return
	 */
	protected AnnotatedLineDocumentProvider createAnnotatedLineDocumentProvider() {
		return new AnnotatedLineDocumentProvider();
	}

	/**
	 * @param entityCreatorService
	 */
	protected void setEntityCreatorService(IEntityCreatorService entityCreatorService) {
		this.entityCreatorService = entityCreatorService;
	}
	
	protected void setEntityCreator(IEntityCreator entityCreator) {
		this.entityCreator = entityCreator;
	}
		
	/**
	 * @param bulkEditorPersistenceService
	 */
	protected void setPersistenceService(
			IEntityPersistenceService persistenceService) {
		this.persistenceService = persistenceService;	
	}
	
	protected IEntityPersistenceService getPersistenceService() {
		return persistenceService;	
	}

	/**
	 * @param bulkEditorLineDisplay
	 */
	protected void setLineDisplayStrategy(
			ILineDisplayStrategy lineDisplayStrategy) {
		this.lineDisplayStrategy = lineDisplayStrategy;		
	}
	
	@Override
	protected ISourceViewer createSourceViewer(Composite parent,
			IVerticalRuler ruler, int styles) {
		
		fAnnotationAccess= getAnnotationAccess();
		fOverviewRuler= createOverviewRuler(getSharedColors());	
		LineSelectionViewer viewer = new LineSelectionViewer(parent, ruler, getOverviewRuler(),  
						isOverviewRulerVisible(), styles);
//						isOverviewRulerVisible(), styles | SWT.WRAP);
		getSourceViewerDecorationSupport(viewer);
				
		return viewer;
	}
	
	/**
	 * Create an annotated line with an "empty" entity, i.e. using the editor
	 * input's default entity type and a zero-length title cache. 
	 * 
	 * @return
	 */
	public LineAnnotation createAnnotatedLineNewObject() {
		
		// Create new object
		Object entity = ((AnnotatedLineDocumentProvider) getDocumentProvider()).
							getEntityCreator(getEditorInput()).createEntity(null);

		LineAnnotation annotation = createAnnotatedLine(entity);
		if (annotation != null) {
			annotation.markAsNew(true);
		}
		return annotation;
	}
	
	/**
	 * Create an annotated line, first creating an entity of type "key" - this key
	 * must be recognized by the editor's entity creator.
	 *  
	 * @param key
	 * @param value
	 * @return 
	 */
	public LineAnnotation createAnnotatedLineNewObject(Object key, String titleCache) {
		
		// Create new object
		Object entity = ((AnnotatedLineDocumentProvider) getDocumentProvider()).
							getEntityCreator(getEditorInput()).createEntity(key, titleCache);

		LineAnnotation annotation = createAnnotatedLine(entity);
		if (annotation != null) {
			annotation.markAsNew(true);
		}
		return annotation;
		
	}
	
	/**
	 * Creates an annotated line at the end of the document. The annotation contains the entity.
	 * 
	 * @param entity
	 * @return
	 */
	public LineAnnotation createAnnotatedLine(Object entity) {
		
		IEditorInput input = getEditorInput();
		AnnotatedLineDocumentProvider provider = (AnnotatedLineDocumentProvider) getDocumentProvider();
		
		LineAnnotation annotation = null;
		try {
			annotation = provider.createAnnotatedLine(input, entity);

			// Jump to new line
			int start= provider.getAnnotationModel(input).getPosition(annotation).getOffset();
			selectAndReveal(start, 0);
			
		} catch (BadLocationException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}		
		return annotation;
	}
	
	/**
	 * @param line
	 */
	public void removeAnnotatedLine(int lineno) {
		((AnnotatedLineDocumentProvider) getDocumentProvider()).removeAnnotatedLine(getEditorInput(), lineno);
	}

	/**
	 * @param annotation
	 */
	public void removeAnnotatedLine(LineAnnotation annotation) {
		((AnnotatedLineDocumentProvider) getDocumentProvider()).removeAnnotatedLine(getEditorInput(), annotation);
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.ui.texteditor.AbstractTextEditor#doSave(org.eclipse.core.runtime.IProgressMonitor)
	 */
	@Override
	public void doSave(IProgressMonitor progressMonitor) {
		if (getConversationHolder() != null) {
			if( ! getConversationHolder().isBound()){
				getConversationHolder().bind();
			}
			super.doSave(progressMonitor);
			getConversationHolder().commit(true);
		} else {
			super.doSave(progressMonitor);	
		}
		firePropertyChange(PROP_DIRTY);
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.ui.texteditor.AbstractTextEditor#setFocus()
	 */
	@Override
	public void setFocus() {
		super.setFocus();
		if (getConversationHolder() != null) {
			getConversationHolder().bind();
		}
		// TODO pass focus to underlying widgets
	}

	/* (non-Javadoc)
	 * @see eu.etaxonomy.cdm.api.conversation.IConversationEnabled#getConversationHolder()
	 */
	public ConversationHolder getConversationHolder() {
		return conversation;
	}

	/* (non-Javadoc)
	 * @see eu.etaxonomy.cdm.persistence.hibernate.ICdmPostDataChangeObserver#update(eu.etaxonomy.cdm.persistence.hibernate.CdmDataChangeMap)
	 */
	public void update(CdmDataChangeMap changeEvents) {}

	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.operations.IPostOperationEnabled#postOperation(eu.etaxonomy.cdm.model.common.CdmBase)
	 */
	public boolean postOperation(CdmBase objectAffectedByOperation) {
		return false;
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.ui.editors.text.TextEditor#dispose()
	 */
	@Override
	public void dispose() {
		super.dispose();
		conversation.close();
	}
}