/**
 * Copyright (C) 2007 EDIT
 * European Distributed Institute of Taxonomy 
 * http://www.e-taxonomy.eu
 * 
 * The contents of this file are subject to the Mozilla Public License Version 1.1
 * See LICENSE.TXT at the top of this package for the full license terms.
 */

package eu.etaxonomy.taxeditor.editor.name;

import java.util.Iterator;

import org.apache.log4j.Logger;
import org.eclipse.core.commands.operations.IUndoContext;
import org.eclipse.core.runtime.Assert;
import org.eclipse.jface.text.Document;
import org.eclipse.jface.text.IUndoManager;
import org.eclipse.jface.text.IUndoManagerExtension;
import org.eclipse.jface.text.Position;
import org.eclipse.jface.text.TextViewerUndoManager;
import org.eclipse.jface.text.source.Annotation;
import org.eclipse.jface.text.source.AnnotationModel;
import org.eclipse.jface.text.source.AnnotationPainter;
import org.eclipse.jface.text.source.IAnnotationAccess;
import org.eclipse.jface.text.source.IVerticalRuler;
import org.eclipse.jface.text.source.SourceViewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Layout;
import org.eclipse.ui.IActionBars;
import org.eclipse.ui.IEditorSite;
import org.eclipse.ui.actions.ActionFactory;
import org.eclipse.ui.forms.widgets.TableWrapData;
import org.eclipse.ui.forms.widgets.TableWrapLayout;
import org.eclipse.ui.operations.OperationHistoryActionHandler;
import org.eclipse.ui.operations.RedoActionHandler;
import org.eclipse.ui.operations.UndoActionHandler;
import org.eclipse.ui.texteditor.DefaultMarkerAnnotationAccess;
import org.eclipse.ui.texteditor.IWorkbenchActionDefinitionIds;

import eu.etaxonomy.cdm.model.name.TaxonNameBase;
import eu.etaxonomy.cdm.model.taxon.Taxon;
import eu.etaxonomy.cdm.strategy.parser.ParserProblem;
import eu.etaxonomy.taxeditor.editor.EditorAnnotation;
import eu.etaxonomy.taxeditor.editor.LineWrapSquigglesStrategy;
import eu.etaxonomy.taxeditor.editor.ViewerConfiguration;
import eu.etaxonomy.taxeditor.model.Resources;

/**
 * SourceViewer implementation called by NameComposite.
 *  
 * @author p.ciardelli
 * @created 27.05.2008
 * @version 1.0
 */
public class NameViewer extends SourceViewer {
	private static final Logger logger = Logger
			.getLogger(NameViewer.class);
		
	public static final int RULER_WIDTH = 16;
	
	private IVerticalRuler ruler;
	private AnnotationModel annotationModel;

	public NameViewer(Composite parent) {
		super(parent, new RulerWithIcon(RULER_WIDTH), SWT.WRAP | SWT.MULTI | SWT.RESIZE);
											
		this.ruler = getVerticalRuler();
				
		setBackground(Resources.getColor(Resources.COLOR_COMPOSITE_BACKGROUND));
				
		// Lay out the viewer's widgets
		getControl().setLayoutData(new TableWrapData(TableWrapData.FILL_GRAB));
		TableWrapData twdata = new TableWrapData(TableWrapData.FILL_GRAB);
		getTextWidget().setLayoutData(twdata);
		getTextWidget().setLineSpacing(5);
		
		// Default implementation adds ruler after the text widget, then 
		//	uses a custom layout to display it before the text widget
		ruler.getControl().moveAbove(getTextWidget());

		annotationModel = new AnnotationModel();
		this.setDocument(new Document(""), annotationModel);
		
		createAnnotationPainter();

		this.configure(new ViewerConfiguration());		
		
//		setDecorationSupport();
//		DocumentUndoManagerRegistry.connect(this.getDocument());
//		IDocumentUndoManager docUndoManager = DocumentUndoManagerRegistry.getDocumentUndoManager(this.getDocument());
		
		/**
		getEditorSite().getActionBars().setGlobalActionHandler(ActionFactory.UNDO.getId(), new UndoActionHandler(getSite(), undoContext));
		getEditorSite().getActionBars().setGlobalActionHandler(ActionFactory.REDO.getId(), new RedoActionHandler(getSite(), undoContext));		
		**/
	}
	
	public void setBackground(Color color) {
		// Set background color of ruler
		ruler.getControl().setBackground(color);

		// Set background color of text widget
		getTextWidget().setBackground(color);
		
		// Set background color of strip between ruler and textWidget
		getTextWidget().getParent().setBackground(color); 
	}
	
	public Control getRulerControl() {
		return ruler.getControl();
	}

	public void setIcon(Image icon) {
		if (ruler instanceof RulerWithIcon) {
			((RulerWithIcon) ruler).setIcon(icon);
		} else {
			logger.warn("Viewer's IVerticalRuler is not an instance of RulerWithIcon.");
		}
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.jface.text.source.SourceViewer#createLayout()
	 */
	protected Layout createLayout() {
		TableWrapLayout layout = new TableWrapLayout();
		layout.numColumns = 2;
		layout.verticalSpacing = 0;
		return layout;
	}
		
	private void createAnnotationPainter(){
		// Annotations section
		IAnnotationAccess fAnnotationAccess = new DefaultMarkerAnnotationAccess();
		
		// To paint the annotations
		AnnotationPainter annotationPainter = new AnnotationPainter(this, fAnnotationAccess);

		// Default SquigglesStrategy doesn't recognize line wraps
		annotationPainter.addDrawingStrategy(LineWrapSquigglesStrategy.ID, new LineWrapSquigglesStrategy());
		
		// Add ability to paint red squigglies
		annotationPainter.addAnnotationType(EditorAnnotation.ERROR_TYPE, LineWrapSquigglesStrategy.ID);
		annotationPainter.setAnnotationTypeColor(EditorAnnotation.ERROR_TYPE, 
				new Color(Display.getDefault(), EditorAnnotation.ERROR_RGB));

		// Add ability to paint yellow squigglies
		annotationPainter.addAnnotationType(EditorAnnotation.WARNING_TYPE, LineWrapSquigglesStrategy.ID);
		annotationPainter.setAnnotationTypeColor(EditorAnnotation.WARNING_TYPE, 
				new Color(Display.getDefault(), EditorAnnotation.WARNING_RGB));
		
		this.addPainter(annotationPainter);
	}


	public void clearErrors() {
		Iterator<Annotation> annotations = this.getAnnotationModel().getAnnotationIterator();
		while (annotations.hasNext()) {
			Annotation annotation = annotations.next(); 
			if (annotation.getType().equals(EditorAnnotation.ERROR_TYPE) || annotation.getType().equals(EditorAnnotation.WARNING_TYPE)) {
				this.getAnnotationModel().removeAnnotation(annotation);
			}
		}
	}
	
	/**
	 * If <code>name.hasProblem()</code> is <code>true</code>, underlines section
	 * of text bounded by <code>name.getProblemStarts()</code> and 
	 * <code>name.getProblemEnds()</code>.
	 * 
	 * @param name
	 */
	public void setShowParsingErrors(TaxonNameBase<?, ?> name) {
				
		String text = this.getTextWidget().getText();
						
		if (name.hasProblem() && text.length() > 0) {
			int start = name.getProblemStarts();
			int length = name.getProblemEnds() - start;
			
			if (start == -1 || name.getProblemEnds() == -1) {
				return;
			}
			
			// Don't let squigglies try to draw beyond the end of the text
			if (text.length() < start + length) {
				length = text.length() - start;
			}
			
			for (ParserProblem problem : name.getParsingProblems()) {
				
				String type = null;
				if (problem.isWarning()) {
					type = EditorAnnotation.WARNING_TYPE;
				}
				if (problem.isError()) {
					type = EditorAnnotation.ERROR_TYPE;
				}
				if (type == null) {
					continue;
				}
				this.getAnnotationModel().addAnnotation(
						new EditorAnnotation(type, 0, problem.getMessage()), 
						new Position(start, length));
			}
		}
	}
	
	
	public void setShowSecError(Taxon taxon) {
		
		// If taxon has no sec, show an annotation
		if (taxon.getSec() == null) {
			
			String text = "This taxon requires a sec. reference.";
			
			this.getAnnotationModel().addAnnotation(
					new EditorAnnotation(0, text),
					new Position(0, 0));
		}
	}

			
	public void setText(String text) {
		if (text == null) {
			text = "";
		}
		try {
			Assert.isNotNull(text);
			// TODO figure out why getTextWidget() returns null!
			if (this.getTextWidget() == null) {
				return;
			}
			Assert.isNotNull(this.getTextWidget());
			this.getTextWidget().setText(text);
		} catch (RuntimeException e) {
			e.printStackTrace();
		}
	}

	public void setCursorToEOL() {
		getTextWidget().setCaretOffset(getTextWidget().getText().length());
		getTextWidget().setFocus();
	}

	public void createUndoSupport(IEditorSite editorSite) {

		IUndoManager undoManager = new TextViewerUndoManager(25);
		this.setUndoManager(undoManager);
		undoManager.connect(this);
		
//		IUndoContext workbenchUndoContext = UiUtil.getWorkbenchUndoContext();
		
		IUndoContext workbenchUndoContext = ((IUndoManagerExtension)undoManager).getUndoContext();
		
		OperationHistoryActionHandler undoAction = new UndoActionHandler(editorSite, workbenchUndoContext);
//		undoAction.setActionDefinitionId(IWorkbenchActionDefinitionIds.UNDO);
		undoAction.setActionDefinitionId(ActionFactory.UNDO.getId());
		
		// Create the redo action.
		OperationHistoryActionHandler redoAction = new RedoActionHandler(editorSite, workbenchUndoContext);
//		redoAction.setActionDefinitionId(IWorkbenchActionDefinitionIds.REDO);
		undoAction.setActionDefinitionId(ActionFactory.REDO.getId());

		IActionBars actionBars = editorSite.getActionBars();
		if (actionBars != null) {
//				actionBars.setGlobalActionHandler(IWorkbenchActionDefinitionIds.UNDO, undoAction);
//				actionBars.setGlobalActionHandler(IWorkbenchActionDefinitionIds.REDO, redoAction);
			actionBars.setGlobalActionHandler(ActionFactory.UNDO.getId(), undoAction);
			actionBars.setGlobalActionHandler(ActionFactory.REDO.getId(), redoAction);
			
		}		
		
//		actionBars.setGlobalActionHandler(ActionFactory.UNDO.getId(), undoAction);
//		actionBars.setGlobalActionHandler(ActionFactory.REDO.getId(), redoAction);
	}
	
	public void createUndoSupport_(IEditorSite editorSite) {
		IUndoManager undoManager = new TextViewerUndoManager(25);
		this.setUndoManager(undoManager);
		undoManager.connect(this);
		IUndoContext undoContext;
		if (undoManager instanceof IUndoManagerExtension) {
			undoContext = ((IUndoManagerExtension)undoManager).getUndoContext();
		
			OperationHistoryActionHandler undoAction = new UndoActionHandler(editorSite, undoContext);
//			PlatformUI.getWorkbench().getHelpSystem().setHelp(undoAction, IAbstractTextEditorHelpContextIds.UNDO_ACTION);
			undoAction.setActionDefinitionId(IWorkbenchActionDefinitionIds.UNDO);
			
			// Create the redo action.
			OperationHistoryActionHandler redoAction = new RedoActionHandler(editorSite, undoContext);
//			PlatformUI.getWorkbench().getHelpSystem().setHelp(redoAction, IAbstractTextEditorHelpContextIds.REDO_ACTION);
			redoAction.setActionDefinitionId(IWorkbenchActionDefinitionIds.REDO);


			IActionBars actionBars = editorSite.getActionBars();
			if (actionBars != null) {
				actionBars.setGlobalActionHandler(IWorkbenchActionDefinitionIds.UNDO, undoAction);
				actionBars.setGlobalActionHandler(IWorkbenchActionDefinitionIds.REDO, redoAction);
				
			}
		}
	}
}

