/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.navigation;

import java.util.HashSet;
import java.util.Set;
import java.util.UUID;

import org.apache.log4j.Logger;
import org.eclipse.core.commands.ExecutionEvent;
import org.eclipse.core.commands.operations.IUndoContext;
import org.eclipse.core.commands.operations.UndoContext;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.jface.viewers.TreeSelection;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.IEditorReference;
import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.handlers.HandlerUtil;
import org.eclipse.ui.navigator.CommonViewer;

import eu.etaxonomy.cdm.model.common.CdmBase;
import eu.etaxonomy.cdm.model.taxon.Synonym;
import eu.etaxonomy.cdm.model.taxon.Taxon;
import eu.etaxonomy.cdm.model.taxon.TaxonNode;
import eu.etaxonomy.taxeditor.editor.EditorUtil;
import eu.etaxonomy.taxeditor.editor.TaxonEditorInput;
import eu.etaxonomy.taxeditor.editor.internal.TaxeditorEditorPlugin;
import eu.etaxonomy.taxeditor.model.AbstractUtility;
import eu.etaxonomy.taxeditor.navigation.internal.TaxeditorNavigationPlugin;
import eu.etaxonomy.taxeditor.navigation.navigator.TaxonNavigator;

/**
 * @author n.hoffmann
 * @created 24.03.2009
 * @version 1.0
 */
public class NavigationUtil extends AbstractUtility{
	private static final Logger logger = Logger.getLogger(NavigationUtil.class);
	private static IUndoContext defaultUndoContext;
	
	/**
	 * FIXME Introduces a dependency on eu.etaxonomy.taxeditor.editor. This is highly undesirable.
	 * 
	 * It is not clear at the moment by what mechanics we are going to open the 
	 * editor. I would prefer some sort of resource model that has the editor linked 
	 * to the resource type of taxonbase. Another reason to switch to CommonNavigatorFamework
	 * 
	 * @param uuid
	 */
	public static void openEditor(TaxonNode selection) {
		try {
			UUID entityUuid = selection.getUuid();			
			EditorUtil.open(entityUuid);
		} catch (PartInitException e) {
			logger.error("Error opening the editor", e);
		}
	}
	
	public static Shell getShell() {
		return getActiveWindow().getShell();
	}

	public static IWorkbenchWindow getActiveWindow() {
		return TaxeditorNavigationPlugin.getDefault().getWorkbench().
				getActiveWorkbenchWindow();
	}
	
	public static void openEmpty(UUID parentNodeUuid) {
		try {
			EditorUtil.openEmpty(parentNodeUuid);
		} catch (PartInitException e) {
			logger.error("Error opening the editor", e);
		}
	}
	
	public static IUndoContext getWorkbenchUndoContext() {
		return TaxeditorEditorPlugin.getDefault().getWorkbench().
					getOperationSupport().getUndoContext();
	}

	public static IUndoContext getUndoContext() {
		// FIXME this has to be more specific. Every widget has to have its own undo context
//		return IOperationHistory.GLOBAL_UNDO_CONTEXT;
		
		// Plug-ins that wish their operations to be undoable from workbench views 
		// such as the Navigator or Package Explorer should assign the workbench 
		// undo context to their operations. 
		if (defaultUndoContext == null) {
			defaultUndoContext = new UndoContext();
		}
		return defaultUndoContext;
	}

	/**
	 * Whether a taxonNode has unsaved changes.
	 * 
	 * @param taxonNode
	 * @return
	 */
	public static boolean isDirty(TaxonNode taxonNode){
		
		for (IEditorReference reference : getActivePage().getEditorReferences()) {
			
			try {
				if (reference.getEditorInput() instanceof TaxonEditorInput) {
					TaxonEditorInput editorInput = (TaxonEditorInput) reference.getEditorInput();
					if(editorInput.getTaxonNode().equals(taxonNode) && reference.isDirty()){
						return true;
					}
				}
			} catch (PartInitException e) {
				logger.error(e);
				throw new RuntimeException(e);
			}
			
		}
		return false;
	}

	/**
	 * 
	 * @param element
	 * @param parentElement
	 */
	public static void selectInNavigator(Object element, Object parentElement) {
		TaxonNavigator navigator = getNavigator();
		if (navigator != null) {
			CommonViewer viewer = navigator.getCommonViewer();
			if (viewer != null) {
				if (parentElement != null) {
					viewer.setExpandedState(parentElement, true);
				}
				viewer.setSelection(new StructuredSelection((TaxonNode) element));
			}
		}
	}

	/**
	 * @param selection
	 */
	public static void openSearch(Object selection) {
		if(selection instanceof Taxon){
			Taxon taxon = (Taxon) selection;
			
			handleOpeningOfMultipleTaxonNodes(taxon.getTaxonNodes());
			
		}else if(selection instanceof Synonym){
			Synonym synonym = (Synonym) selection;
			
			Set<TaxonNode> taxonNodes = new HashSet<TaxonNode>();
			for (Taxon taxon : synonym.getAcceptedTaxa()){
				taxonNodes.addAll(taxon.getTaxonNodes());
			}
			handleOpeningOfMultipleTaxonNodes(taxonNodes);
			
		}else{
			warningDialog("Not implemented yet", "You chose to open a name that has no connection to a taxa. The Editor does not support editing of such a content type at the moment.");
			logger.warn("Could not open editor for selection. Selection was of type: " + selection.getClass().getSimpleName());
		}
		
	}

	/**
	 * @param taxonNodes
	 */
	private static void handleOpeningOfMultipleTaxonNodes(
			Set<TaxonNode> taxonNodes) {
		
		if(taxonNodes.size() == 1){
			openEditor(taxonNodes.iterator().next());
		}else if(taxonNodes.size() > 1){
			// FIXME implement a dialog that shows all possible taxa and let the user choose which he wants to open.
			warningDialog("Not implemented yet", "The accepted taxon is in multiple taxonomic trees. We currently do not know which one you want to open." +
					" This case is not handled yet by the software.");
		}else if(taxonNodes.size() == 0){
			// this is an undesired state
			warningDialog("Incorrect state", "The accepted taxon is not in a taxonomic view. This should not have happened.");
		}
	}

	/**
	 * @return the TaxonNavigator instance if present
	 */
	public static TaxonNavigator getNavigator() {
		return (TaxonNavigator) TaxeditorEditorPlugin.getDefault().getWorkbench().
					getActiveWorkbenchWindow().getActivePage().findView(TaxonNavigator.ID);
	}
}