/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.store;

import java.util.ArrayList;
import java.util.List;

import org.apache.log4j.Logger;
import org.eclipse.core.runtime.Status;
import org.eclipse.ui.PlatformUI;
import org.springframework.core.io.ClassPathResource;
import org.springframework.core.io.Resource;
import org.springframework.security.providers.ProviderManager;

import eu.etaxonomy.cdm.api.application.CdmApplicationController;
import eu.etaxonomy.cdm.api.conversation.ConversationHolder;
import eu.etaxonomy.cdm.api.service.IAgentService;
import eu.etaxonomy.cdm.api.service.ICommonService;
import eu.etaxonomy.cdm.api.service.ILocationService;
import eu.etaxonomy.cdm.api.service.INameService;
import eu.etaxonomy.cdm.api.service.IReferenceService;
import eu.etaxonomy.cdm.api.service.ITaxonService;
import eu.etaxonomy.cdm.api.service.ITaxonTreeService;
import eu.etaxonomy.cdm.api.service.ITermService;
import eu.etaxonomy.cdm.api.service.IUserService;
import eu.etaxonomy.cdm.api.service.IVocabularyService;
import eu.etaxonomy.cdm.api.service.config.IIdentifiableEntityServiceConfigurator;
import eu.etaxonomy.cdm.api.service.config.ITaxonServiceConfigurator;
import eu.etaxonomy.cdm.database.DataSourceNotFoundException;
import eu.etaxonomy.cdm.database.DatabaseSchemaMismatchException;
import eu.etaxonomy.cdm.database.DbSchemaValidation;
import eu.etaxonomy.cdm.database.ICdmDataSource;
import eu.etaxonomy.cdm.ext.IEditGeoService;
import eu.etaxonomy.cdm.model.agent.AgentBase;
import eu.etaxonomy.cdm.model.agent.TeamOrPersonBase;
import eu.etaxonomy.cdm.model.common.IdentifiableEntity;
import eu.etaxonomy.cdm.model.common.Language;
import eu.etaxonomy.cdm.model.name.TaxonNameBase;
import eu.etaxonomy.cdm.model.reference.ReferenceBase;
import eu.etaxonomy.taxeditor.datasource.CdmDataSourceRepository;
import eu.etaxonomy.taxeditor.dialogs.DatasourceLauncherDialog;
import eu.etaxonomy.taxeditor.io.ExportHandler;
import eu.etaxonomy.taxeditor.io.ImportHandler;
import eu.etaxonomy.taxeditor.preference.PreferencesUtil;
import eu.etaxonomy.taxeditor.store.internal.TaxeditorStorePlugin;

/**
 * This implementation of ICdmDataRepository depends on hibernate sessions to store the data correctly 
 * for the current session. No state is held in this class.
 * 
 * Only methods that either get or manipulate data are exposed here. So this class acts as a facade 
 * for the methods in cdmlib-service. 
 * 
 * 
 * @author n.hoffmann
 * @created 17.03.2009
 * @version 1.0
 */
public class CdmStore{
	private static final Logger logger = Logger.getLogger(CdmStore.class);
	
	// FIXME change this to ClassPathResources as soon as it is included into the plugin
	private static final Resource DEFAULT_APPLICATION_CONTEXT = new ClassPathResource("/eu/etaxonomy/cdm/editorApplicationContext.xml", TaxeditorStorePlugin.class.getClassLoader());
	private static final DbSchemaValidation DEFAULT_DB_SCHEMA_VALIDATION = DbSchemaValidation.UPDATE;
	
	private static CdmStore instance;
		
	private CdmApplicationController applicationController;

	private static LoginManager loginManager;
	
	private static ImportHandler importHandler;
	
	private static ExportHandler exportHandler;
	
	private Language language;
	
	private static ICdmDataSource cdmDatasource;
	
	
	/**
	 * 
	 * @param applicationContextBean
	 * @return
	 */
	protected static CdmStore getDefault(){
		if(instance == null){
			
			// Prompt user for datasource if there are more than one
			if(PreferencesUtil.getPreferenceStore().getBoolean(PreferencesUtil.SHOW_DATASOURCE_CHOOSER) 
				&& CdmDataSourceRepository.getAll().size() > 1){
				createDatasourceChooser(null);
			}else{
				// Start default cdm database
				instance = initialize(DEFAULT_DB_SCHEMA_VALIDATION, DEFAULT_APPLICATION_CONTEXT);
			}
		
		}
		
		return instance;
	}
	
	private static CdmStore initialize(DbSchemaValidation dbSchemaValidation, Resource applicationContextBean){	
		
		logger.info("Initializing application context ...");
		
		cdmDatasource = CdmDataSourceRepository.getCurrentDataSource();
		
		try {
			if (!cdmDatasource.testConnection()) {
				createDatasourceChooser("The database for the chosen datasource '" + cdmDatasource + "' \n could not be connected to." +
						"Please choose a new data source to connect to");
			} else {
				instance = new CdmStore(cdmDatasource, dbSchemaValidation, applicationContextBean);
				logger.info("Application context initialized.");
				
				if (!getCommonService().isDatabaseSchemaCompatible()) {
					// Show the datasource chooser
					if(CdmDataSourceRepository.getAll().size() > 1){
						createDatasourceChooser("The database schema for the chosen " +
								"datasource '" + cdmDatasource + "' \n is not valid for this version of the taxonomic editor. \n" +
								"Please choose a new data source to connect to");
					} else {
						throw new DatabaseSchemaMismatchException();
					}
				}
			}
			
			return instance;
		} catch (DataSourceNotFoundException e) {
			StoreUtil.errorDialog("Chosen Datasource is not available", "Could not connect to the chosen " +
					"datasource '" + cdmDatasource + "'. Please check settings in datasources.xml. If the datasource " +
							"is located on a remote machine, make sure you are connected to the network.");
			logger.error(e);
		} catch (NullPointerException e){
			StoreUtil.errorDialog("Corrupt Datasource", "The configuration for the chosen " +
					"datasource '" + cdmDatasource + "' is corrupt. Please check settings in datasources.xml.");
			logger.error(e);
		} catch (DatabaseSchemaMismatchException e){
			StoreUtil.errorDialog("Database Schema incompatible", "The database schema for the chosen " +
					"datasource '" + cdmDatasource + "' is not valid for this version of the taxonomic editor.");
			logger.error(e);
		} catch (Exception e){
			StoreUtil.errorDialog("Unknown Datasource Error", "An error occurred while connecting to the datasource." +
					"Please refer to the error log.");
			logger.error(e);
		} 
		
		// FIXME present datasource chooser dialog instead of closing the workbench
		PlatformUI.getWorkbench().close();
		return null;
	}

	/**
	 * Creates the datasource chooser.
	 * @param message Message to show. Specify null if none.
	 */
	private static void createDatasourceChooser(String message) {
		DatasourceLauncherDialog chooseDataSource = new DatasourceLauncherDialog(StoreUtil.getShell(), message);
		if(chooseDataSource.open() == Status.OK){
			instance = initialize(DEFAULT_DB_SCHEMA_VALIDATION, DEFAULT_APPLICATION_CONTEXT);
		}else{
			// no datasource chosen.
			logger.warn("No datasource chosen. Exiting now.");
			PlatformUI.getWorkbench().close();
		}
	}

	/**
	 * 
	 */
	private CdmStore(ICdmDataSource dataSource, DbSchemaValidation dbSchemaValidation, Resource applicationContextBean) {
		try {
			applicationController = CdmApplicationController.NewInstance(applicationContextBean, dataSource, dbSchemaValidation, false);
		} catch (Exception e) {
			throw new RuntimeException(e);
		}
	}
	
	/**
	 * All calls to the datastore require 
	 * 
	 * @return
	 */
	private CdmApplicationController getApplicationController(){
		try{
			return applicationController;
		}catch(Exception e){
			logger.error("Exception thrown", e);
		}
		return null;
	}
	

	@Deprecated
	public static CdmApplicationController getApplicationControllerWillBeObsolete(){
		return getDefault().getApplicationController();
	}

	/*
	 * CONVERSATIONS
	 */
	
	/**
	 * Create a new conversation and bind resources to it
	 * 
	 * @return
	 */
	public static ConversationHolder NewConversation(){
		
		ConversationHolder conversation = getDefault().getApplicationController().NewConversation();

		return conversation;
	}

	/**
	 * Creates a new conversation, binds resources to the conversation and 
	 * start a transaction for this conversation.
	 * 
	 * @return
	 */
	public static ConversationHolder NewTransactionalConversation() {
		ConversationHolder conversation = NewConversation();
		
		conversation.startTransaction();
		return conversation;
	}
	
	/*
	 * EXPOSING SERVICES
	 */
	
	public static ITaxonService getTaxonService(){ return getDefault().getApplicationController().getTaxonService();}
	
	public static ITaxonTreeService getTaxonTreeService() { return getDefault().getApplicationController().getTaxonTreeService();}
	
	public static INameService getNameService(){ return getDefault().getApplicationController().getNameService();}
	
	public static IReferenceService getReferenceService(){ return getDefault().getApplicationController().getReferenceService();}
	
	public static ILocationService getLocationService(){ return getDefault().getApplicationController().getLocationService();}

	public static ProviderManager getAuthenticationManager() { return getDefault().getApplicationController().getAuthenticationManager();}

	public static IUserService getUserService() { return getDefault().getApplicationController().getUserService(); }
	
	public static ICommonService getCommonService() { return getDefault().getApplicationController().getCommonService(); }
	
	public static IAgentService getAgentService() { return getDefault().getApplicationController().getAgentService(); }
	
	public static ITermService getTermService() { return getDefault().getApplicationController().getTermService(); }
	
	public static IVocabularyService getVocabularyService() { return getDefault().getApplicationController().getVocabularyService(); }
	
	public static IEditGeoService getGeoService(){
		return (IEditGeoService) getDefault().getApplicationController().getBean("editGeoService");
	}
	
	/*
	 * METHODS TO FIND ENTITIES
	 */
	
	/**
	 * 
	 * @param configurator
	 * @return
	 */
	public static List<TaxonNameBase> findNames(IIdentifiableEntityServiceConfigurator configurator){
		// TODO we want to use IIdentifiableEntityServiceConfigurator for all find methods
		// unfortunately this is not consistently implemented in the library.
		// FIXME use proper method once it is implemented in the library
		String titleSearchString = configurator.getTitleSearchString().replace("*", "%");
		
		return getNameService().getNamesByName(titleSearchString);
	}

	/**
	 * 
	 * @param configurator
	 * @return
	 */
	public static List<IdentifiableEntity> findTaxaAndNames(ITaxonServiceConfigurator configurator){
		return getTaxonService().findTaxaAndNames(configurator).getRecords();
	}
	
	/**
	 * 
	 * @param configurator
	 * @return
	 */
	public static List<ReferenceBase> findReferences(IIdentifiableEntityServiceConfigurator configurator){
		// TODO we want to use IIdentifiableEntityServiceConfigurator for all find methods
		// unfortunately this is not consistently implemented in the library.
		// FIXME use proper method once it is implemented in the library
		String titleSearchString = configurator.getTitleSearchString().replace("*", "%");
		
		return getReferenceService().findByTitle(null, titleSearchString, null, null, null, null, null, null).getRecords();
	}
	
	/**
	 * 
	 * @param configurator
	 * @return
	 */
	public static List<AgentBase> findAgents(IIdentifiableEntityServiceConfigurator configurator){
		// TODO we want to use IIdentifiableEntityServiceConfigurator for all find methods
		// unfortunately this is not consistently implemented in the library.
		// FIXME use proper method once it is implemented in the library
		String titleSearchString = configurator.getTitleSearchString().replace("*", "%");
		
		return getAgentService().findByTitle(null, titleSearchString, null, null, null, null, null, null).getRecords();
	}

	public static List<TeamOrPersonBase> findTeamOrPersons(IIdentifiableEntityServiceConfigurator configurator){
		// TODO move this to cdmlib
		List<TeamOrPersonBase> result = new ArrayList<TeamOrPersonBase>();
		for (AgentBase agent : findAgents(configurator)) {
			if (agent instanceof TeamOrPersonBase) {
				result.add((TeamOrPersonBase) agent);
			}
		}
		return result;
	}
	
	/*
	 * LANGUAGE
	 */
	
	/** 
	 * @return
	 */
	public static Language getDefaultLanguage(){
		if(getDefault().getLanguage() == null){
			getDefault().setLanguage(Language.DEFAULT());
		}
		return getDefault().getLanguage();
	}

	public static void setDefaultLanguage(Language language){
		getDefault().setLanguage(language);
	}
	
	/**
	 * @return the language
	 */
	private Language getLanguage() {
		return language;
	}

	/**
	 * @param language the language to set
	 */
	private void setLanguage(Language language) {
		this.language = language;
	}
	
	/*
	 * LOGIN
	 */
	
	/**
	 * 
	 */
	public static LoginManager getLoginManager(){
		if(loginManager == null){
			loginManager = new LoginManager();
		}
		return loginManager;
	}
	
	/*
	 * IMPORT/EXPORT FACTORIES
	 */
	
	/**
	 * 
	 */
	public static synchronized ImportHandler getImportHandler(){
		if(importHandler == null){
			importHandler = ImportHandler.NewInstance(getDefault().getApplicationController());
		}
		return importHandler;
	}
	
	/**
	 * 
	 * @return
	 */
	public static synchronized ExportHandler getExportHandler(){
		if(exportHandler == null){
			exportHandler = ExportHandler.NewInstance(getDefault().getApplicationController());
		}
		return exportHandler;
	}
}
